@@ -1,4 +1,4 @@
-//TrackView.cpp: Implementation of the TrackView class (default playlist / search result / album view).
+//TrackView.cpp: Implementation of the TrackView class (view used for  playlists / search results).
 
 #include "TrackView.h"
 
@@ -7,14 +7,12 @@ TrackView::TrackView(Spotify::Session* pSession, QWidget* parent) :
 {
         //constructs the UI track view
 
-        QStringList sortByList; //list of sorting by alternatives
-
-	setAttribute(Qt::WA_Maemo5StackedWindow);
+        setAttribute(Qt::WA_Maemo5StackedWindow); //set flag that window is part of window stack
 
         m_pSession = pSession; //set the Spotify session instance
+        connect(m_pSession,SIGNAL(albumBrowserReady(Spotify::AlbumBrowser*,qint32)),SLOT(OnAlbumBrowseReady(Spotify::AlbumBrowser*,qint32)));
 
-        //init the play queue instance
-        m_pQueueView = new QueueView(m_pSession,this);
+        m_CallerID = TRACKVIEW_ID;
 
         //init the track listview instance
         m_pTrackListView = new CListView(this);
@@ -24,22 +22,11 @@ TrackView::TrackView(Spotify::Session* pSession, QWidget* parent) :
 
         m_pTrackListView->setItemDelegate(m_pTrackListDelegate); //connect the delegate to view
         m_pTrackListView->setModel(m_pTrackListModel); //connect the model to view.
-
-        connect(m_pQueueView,SIGNAL(playQueuePosUpdated(qint32,qint32)),SLOT(OnPlayQueuePosUpdated(qint32,qint32)));
         connect(m_pTrackListView,SIGNAL(activated(QModelIndex)),SLOT(OnTrackActivated(QModelIndex)));
 
-        //set up the the track view menu
-        m_pSortByAct = new PickSelectorAction(this);
-        m_pSortByAct->setText(tr("Sort tracks by"));
+        setupContextMenu(); //set up the listview context menu
 
-        m_PlaylistSortByList << tr("Playlist order") << tr("Most popular") << tr("Track name (ascending)") << tr("Track name (descending)") << tr("Artist name (ascending)")
-                   << tr("Artist name (descending)") << tr("Album name (ascending)") << tr("Album name (descending)");
-
-        m_SearchSortByList << tr("Artist, album, track (ascending)") << tr("Most popular") << tr("Track name (ascending)") << tr("Track name (descending)") << tr("Artist name (ascending)")
-                   << tr("Artist name (descending)") << tr("Album name (ascending)") << tr("Album name (descending)");
-
-        connect(m_pSortByAct,SIGNAL(selectionChanged(int)),SLOT(OnSortingSelectionChanged(int)));
-        menuBar()->addAction(m_pSortByAct);
+        setupMenubar(); //set up the view menubar
 
         m_Layout.addWidget(m_pTrackListView);
 
@@ -53,25 +40,140 @@ TrackView::~TrackView()
 {
 }
 
-bool TrackView::getQueueViewUpdateBufPos()
+void TrackView::setupContextMenu()
 {
-    //returns the update buffer position status for the TrackView owned QueueView instance
-    if(m_pQueueView)
+    //set up the listview context menu
+
+    m_pTrackListView->setContextMenuPolicy(Qt::ActionsContextMenu); //actions context menu associated with listview
+
+    //set up the listview contextmenu options
+    QAction* pPlayAlbumAct = new QAction(tr("Play Album"),this);
+    connect(pPlayAlbumAct,SIGNAL(triggered()),SLOT(OnPlayAlbumAction()));
+    m_pTrackListView->addAction(pPlayAlbumAct);
+    QAction* pViewAlbumAct = new QAction(tr("Browse Album"),this);
+    connect(pViewAlbumAct,SIGNAL(triggered()),SLOT(OnViewAlbumAction()));
+    m_pTrackListView->addAction(pViewAlbumAct);
+    QAction* pViewArtistAct = new QAction(tr("Browse Artist"),this);
+    pViewArtistAct->setEnabled(false);
+    //connect(pPlayAlbumAct,SIGNAL(triggered()),SLOT(OnViewArtistAction()));
+    m_pTrackListView->addAction(pViewArtistAct);
+}
+
+void TrackView::setupMenubar()
+ {
+     //set up the view menubar
+
+    //set up the the track view menu
+    m_pSortByAct = new PickSelectorAction(this);
+    m_pSortByAct->setText(tr("Sort tracks by"));
+
+    m_PlaylistSortByList << tr("Playlist order") << tr("Most popular") << tr("Track name (ascending)") << tr("Track name (descending)") << tr("Artist name (ascending)")
+               << tr("Artist name (descending)") << tr("Album name (ascending)") << tr("Album name (descending)");
+
+    m_SearchSortByList << tr("Artist, album, track (ascending)") << tr("Most popular") << tr("Track name (ascending)") << tr("Track name (descending)") << tr("Artist name (ascending)")
+               << tr("Artist name (descending)") << tr("Album name (ascending)") << tr("Album name (descending)");
+
+    connect(m_pSortByAct,SIGNAL(selectionChanged(int)),SLOT(OnSortingSelectionChanged(int)));
+    menuBar()->addAction(m_pSortByAct);
+    menuBar()->addAction(tr("FM Transmitter"));
+    connect(menuBar(),SIGNAL(triggered(QAction*)),SLOT(OnMenubarAction(QAction*)));
+ }
+
+Spotify::Track* TrackView::getTrackFromModelIndex(QModelIndex index)
+{
+    //get track instance pointer from model index. Returns NULL if index is invalid.
+
+    Spotify::Track* track = NULL;
+
+    if(index.isValid())
     {
-        return m_pQueueView->getUpdateBufferPos();
+        int trackPos = index.data(Qt::UserRole).toInt();
+
+        if(m_pPlaylist) //track from playlist
+            track = m_pPlaylist->getTrack(trackPos);
+        else //search result
+        {
+            track = m_TrackList.at(trackPos);
+        }
     }
-    else
-        return true;
+
+    return track;
 }
 
-void TrackView::closeEvent(QCloseEvent* evt)
+void TrackView::OnPlayAlbumAction()
 {
-    //Notification received when view closed
-    Q_UNUSED(evt);
-    m_pQueueView->setActive(false,m_pQueueView->getShuffleActive());
-    emit activeQueueViewChanged(false,m_pQueueView->getShuffleActive()); //notify about change (set the active queueview instance to MainWindow owned)
+    //request to play the album of currently selected track
+
+    Spotify::Track* track = getTrackFromModelIndex(m_pTrackListView->currentIndex());
+
+    if(track)
+    {
+        if(track->getAlbum())
+        {
+            if(m_pSession->browse(track->getAlbum(),&m_CallerID)) //request to load album tracks (browse album)
+            {
+                m_pTrackListView->setDisabled(true);
+                QMaemo5InformationBox::information(this,tr("Loading album tracks"));
+                setAttribute(Qt::WA_Maemo5ShowProgressIndicator,true);
+            }
+        }
+    }
 }
 
+void TrackView::OnAlbumBrowseReady(Spotify::AlbumBrowser* browser, qint32 callerID)
+{
+    //notification that album browse request completed
+
+    Spotify::Error error;
+
+    if(callerID==m_CallerID)
+    {
+        if(browser)
+        {
+            error = sp_albumbrowse_error(browser->getAlbumBrowser());
+            if(error.isError())
+            {
+                //error occured (album browse request failed)
+                QMaemo5InformationBox::information(this,tr("Fetching of album tracks failed: ") + error.toString());
+            }
+            else
+            {
+                if(!browser->load())
+                {
+                    QMaemo5InformationBox::information(this,tr("Failed to load tracks!"));
+                }
+                browser->sortByDiscIndex(); //sort tracks in album order
+                emit playQueueSet(browser->getTracks(false)); //only add available (playable) album tracks to the playqueue
+            }
+        }
+    }
+    setAttribute(Qt::WA_Maemo5ShowProgressIndicator,false);
+    m_pTrackListView->setDisabled(false);
+}
+
+void TrackView::OnViewAlbumAction()
+{
+    //Request to show Album View for currently selected album.
+
+    Spotify::Track* track = getTrackFromModelIndex(m_pTrackListView->currentIndex());
+
+    if(track)
+    {
+        if(track->getAlbum())
+        {
+            emit showAlbumView(track->getAlbum());
+        }
+    }
+}
+
+void TrackView::OnMenubarAction(QAction* action)
+{
+
+    if(action->text()==tr("FM Transmitter"))
+    { //request to display the FM Transmitter control panel dialog (plugin)
+        emit showFMTransmitterDialog();
+    }
+}
 
 void TrackView::OnSortingSelectionChanged(int currSelection)
 {
@@ -167,39 +269,15 @@ void TrackView::OnTrackActivated(QModelIndex trackIndex)
 {
     //A track has been activated (e.g., selected) in the track list
 
-    Spotify::Error error;
-    Spotify::Track* track;
+    Q_UNUSED(trackIndex)
 
-     m_pTrackListView->setEnabled(false); //disable table temporarily
+    m_pTrackListView->setEnabled(false); //disable table temporarily
 
-      int trackPos = trackIndex.data(Qt::UserRole).toInt();
+    setAttribute(Qt::WA_Maemo5ShowProgressIndicator,true); //display progress indicator
+    setPlayQueue();
+    setAttribute(Qt::WA_Maemo5ShowProgressIndicator,false); //hide progress indicator
 
-        if(m_pPlaylist) //track from playlist
-            track = m_pPlaylist->getTrack(trackPos);
-        else //search result
-        {
-            track = m_TrackList.at(trackPos);
-        }
-
-        if(track) //we have got a valid track instance
-        {
-            m_pTrackListView->setEnabled(false); //disable table temporarily
-            setAttribute(Qt::WA_Maemo5ShowProgressIndicator,true); //display progress indicator
-            m_pSession->resetPlaybackMembers();
-            QMaemo5InformationBox::information(this,tr("Buffering"));
-            error = m_pSession->play(track); //request to start playback of track
-            if(error.isError())
-            { //error occured
-                QMaemo5InformationBox::information(this,tr("Error starting track playback: ") + error.toString());
-            }
-            else
-            {
-                setPlayQueue(); //set new playqueue, and move to play queue view
-            }
-            setAttribute(Qt::WA_Maemo5ShowProgressIndicator,false); //hide progress indicator
-        }
-
-        m_pTrackListView->setEnabled(true); //re-enable the list view
+    m_pTrackListView->setEnabled(true); //re-enable the list view
 }
 
 
@@ -224,27 +302,6 @@ void TrackView::setPlayQueue()
     }
 
     emit playQueueSet(PlayQueue); //signal that playqueue has been set / updated
-
-    if(m_pQueueView)
-    {
-        m_pQueueView->setNowPlaying(PlayQueue,0,true,0,0,true);
-        m_pQueueView->setActive(true,m_Shuffle);
-        m_pQueueView->setUpdateBufferPos(true); //allow update of buffering progress
-        emit activeQueueViewChanged(true,m_Shuffle); //notify about active queue view change
-        m_pQueueView->show();
-    }
-}
-
-void TrackView::setShuffleActive(bool active)
-{
-    //set whether shuffle mode is active
-    m_Shuffle = active;
-}
-
-void TrackView::OnPlayQueuePosUpdated(qint32 pos,qint32 shufflePos)
-{
-    //emit notification that position in playqueue has been updated
-    emit playQueuePosUpdated(pos,shufflePos);
 }
 
 
@@ -294,6 +351,7 @@ void TrackView::addTrackListModelItem(Spotify::Track* track, int pos)
 
     //appends new track item to the track list model
     QStandardItem *trackItem = new QStandardItem();
+    trackItem->setEditable(false);
     strSubHeader = "";
     trackItem->setData(trackTypeIcon,ListViewDelegate::iconRole);
     trackItem->setData(track->getName(),ListViewDelegate::headerRole);
